#
# Evantide/Web.pm
#
#  web routines
#  evantide, ltd. staff (staff@evantide.com)
#

package Evantide::Web;

use English;
use Exporter;
use FileHandle;

@ISA       = qw(Exporter);
@EXPORT    = qw(inject_vars expand_path cgi_parse %cgi escape_url_comp
		unescape_url_comp Log
		get_state_from_cookie get_state_cookie_header);

#
# inject_vars ($file, $key1, $var1, $key2, $var2, ...)
#
#   Returns contents of $file as a string, replacing all @@$key's with
#   their respective $var's.
#
sub inject_vars
{
    my ($fh, $result, $filepath, $filetext, $leftpos, $var, %vars, @pieces,
	$piece);


    $filepath = shift @_;

    #
    # build hash of substitution vars
    #
    while ($var = shift @_) { $vars {$var} = shift @_; }
    
    #
    # slurp file; loop on substitutions
    #
    $fh = new FileHandle;
    $fh->open ("$filepath") || die "fatal: could not open $filepath";
    $fh->input_record_separator (undef);
    $filetext = <$fh>;
    close ($fh);

    @pieces = split (/(@@\w+)\b/, $filetext);

    foreach $piece (@pieces) {
	if (($var) = ($piece =~ /^@@(\w+)/)) {
	    if    (defined ($vars {$var})) { $result .= $vars {$var}; }
	    elsif ($var !~ /^quiet_/) { $result .= "\@\@UNDEFINED"; }
	}
	else {
	    $result .= $piece;
	}
    }

    return ($result);
}


#
# expand_path ($filepath)
#
#   Returns $filepath's contents as a string
#
sub expand_path
{
    my ($result);

    open (TEXT, "$_[0]") || die "fatal: could not open $_[0]";
    TEXT->input_record_separator (undef);
    $result = <TEXT>;
    close (TEXT);

    return ($result);
}


#
# cgi_parse ()
#
#   Builds %cgi as (form element, value) pairs from url (get) or body (post).
#
sub cgi_parse
{
    my ($i, $key, $val, @in, $in);


    # Read in text
    if ($ENV {'REQUEST_METHOD'} eq "GET") {
	$in = $ENV {'QUERY_STRING'};
    } elsif ($ENV {'REQUEST_METHOD'} eq "POST") {
	read (STDIN, $in, $ENV {'CONTENT_LENGTH'});
    }

    @in = split (/&/, $in);

    foreach $i (0 .. $#in) {
	# Convert pluses to spaces
	$in [$i] =~ s/\+/ /g;

	# Split into key and value.  
	($key, $val) = split (/=/, $in [$i], 2); # splits on the first =.

	# Convert %XX from hex numbers to alphanumeric
	$key =~ s/%(..)/pack("c",hex($1))/ge;
	$val =~ s/%(..)/pack("c",hex($1))/ge;

	# Associate key and value (separate, if necessary, with \0s)
	$cgi {$key} .= "\0" if (defined ($cgi {$key}));
	$cgi {$key} .= $val;
    }
}


#
# escape_url_comp ($string)
#
#   Escapes string for url
#
sub escape_url_comp {
    $_[0] =~ s/([^a-zA-Z0-9_\-.])/uc sprintf("%%%02x",ord($1))/eg;
    return $_[0];
}


#
# unescape_url_comp ($string)
#
#   Unescapes string for url
#
sub unescape_url_comp {
    $_[0] =~ s/%(..)/pack("c",hex($1))/ge;
    return $_[0];
}


#
# Log ($logpath, @statements)
#
#   Logs @statements to $logpath, with appropriate locking.
#
sub Log
{
    my ($logpath, @statements) = @_; my ($logfd); $logfd = 0;

    open ($logfd, ">>$logpath");

    while (flock ($logfd, 2) == 0) { sleep (1); }
    seek  ($logfd, 0, 2);

    print $logfd scalar (localtime), ": ", @statements, "\n";

    flock ($logfd, 8);
    close ($logfd);
}


#
# get_state__from_cookie ()
#
#   Parse state from browser cookie and return state object; returns undef
#   if either the cookie or the state is invalid.
#
sub get_state_from_cookie
{
    my ($oid, $state);

    if (defined ($ENV {'HTTP_COOKIE'})) {
	($oid) = $ENV {'HTTP_COOKIE'} =~ /evoid\s*=\s*(\d+)/;
    }

    if (not defined ($ENV {'HTTP_COOKIE'}) or
	not ($state = new Evantide::State ($oid)))
    {
	return (undef);
    }

    return ($state);
}


#
# get_state_cookie_header ()
#
#
#
sub get_state_cookie_header
{
    ($state, $path, $domain) = @_;
    my ($oid);

    $oid = $state->get_oid ();

    return qq{Set-cookie: evoid=$oid; path=$path; domain=$domain;\n};
}
